@@ -76,8 +76,8 @@
   "Helper routine for converter macro"
   (cond ((func? cmd :penalty 1)
 	 (converter-set-penalty from to (second cmd)))
-        ((func? cmd :require 1)
-	 (if (not ((second cmd))) (converter-remove from to)))
+;;        ((func? cmd :require 1) ;; already handled earlier now 
+;;	 (if (not ((second cmd))) (converter-remove from to)))
         ((func? cmd :option 2)
 	 (converter-define-option from to (second cmd) (third cmd)))
         ((func? cmd :function 1)
@@ -108,23 +108,38 @@
 	 (to (if (string? to*) to* (symbol->string to*))))
     (set! converter-distance (make-ahash-table))
     (set! converter-path (make-ahash-table))
-    (converter-set-penalty from to 1.0)
-    `(for-each (lambda (x) (converter-cmd ,from ,to x))
-	       ,(list 'quasiquote (map converter-sub options)))))
+;; NEW if (:required) clause present but not fulfilled do nothing
+;; this enables to define several possible implementations of a given converter
+;; not presuming on the availability of external tools : the last valid one is retained
+;; (previously the last defined -even if unavailable- erased whatever was already defined)
+    (cond ((and (in? (car (first options)) '(:penalty)) 
+            (in? (car (second options)) '(:require)) 
+            (not (eval (second (second options))))) (noop))
+          ((and (in? (car (first options)) '(:require)) 
+            (not (eval (second (first options))))) (noop))
+          (else (converter-set-penalty from to 1.0) 
+             `(for-each (lambda (x) (converter-cmd ,from ,to x))
+	         ,(list 'quasiquote (map converter-sub options)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Special converters
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define (converter-shell-cmd l from to)
+  (with x (car l)
+    (string-append (if (or (os-win32?) (os-mingw?)) 
+                     (escape-shell (url-concretize (url-resolve-in-path x))) x) " "
+      (converter-shell-cmd-args (cdr l) from to))))
+
+(define (converter-shell-cmd-args l from to)
   (if (null? l) ""
       (with x (car l)
-	(string-append (cond ((== x 'from) (url-concretize from))
-			     ((== x 'to) (url-concretize to))
+	(string-append (cond ((== x 'from) (escape-shell (url-concretize from)))
+			     ((== x 'to) (escape-shell (url-concretize to)))
 			     (else x))
 		       (cond ((and (string? x) (string-ends? x "=")) "")
                              (else " "))
-		       (converter-shell-cmd (cdr l) from to)))))
+		       (converter-shell-cmd-args (cdr l) from to)))))
 
 (define (converter-shell l from to-format opts)
   ;;(display* "converter-shell " l ", " from ", " to-format ", " opts "\n")
@@ -133,10 +148,11 @@
 	 (dsuf (format-default-suffix to-format))
 	 (suf (if (and dsuf (!= dsuf "")) (string-append "." dsuf) ""))
 	 (to (if (and last? dest) dest (url-glue (url-temp) suf)))
-	 (cmd (converter-shell-cmd l from to)))
+	 (cmd (if (or (os-win32?) (os-mingw?)) (escape-shell (converter-shell-cmd l from to))
+             (converter-shell-cmd l from to))))
     ;;(display* "shell: " cmd "\n")
     (system cmd)
-    to))
+    (if (url-exists? to) to #f)))
 
 (define-public (converter-save s opts)
   "Helper routine for define-format macro"
@@ -144,7 +160,7 @@
 	 (dest (assoc-ref opts 'dest))
 	 (to (if (and last? dest) dest (url-temp))))
     (string-save s to)
-    to))
+    (if (url-exists? to) to #f)))
 
 (define-public (converter-load u opts)
   "Helper routine for define-format macro"
@@ -207,15 +223,22 @@
 ;; Actual conversion
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define-public (std-converter-options from to)
+  (or (ahash-ref converter-options (list from to)) '()))
+
 (define (convert-via what from path options)
   ;;(display* "convert-via " what ", " from ", " path ", " options "\n")
-  (if (null? path) what
+  (if (null? path)
+      (with dest (assoc-ref options 'dest)
+        (when (and dest (url? what) (url? dest) (!= dest what))
+          (system-copy what dest))
+        what)
       (with fun (ahash-ref converter-function (list from (car path)))
 	(if fun
 	    (let* ((last? (null? (cdr path)))
 		   (opts1 (acons 'last? last? options))
-		   (opts2 (ahash-ref converter-options (list from (car path))))
-		   (what* (fun what (append opts1 (or opts2 '()))))
+		   (opts2 (std-converter-options from (car path)))
+		   (what* (fun what (append opts1 opts2)))
 		   (result (convert-via what* (car path) (cdr path) options)))
 	      (if (and (not last?) (string-ends? (car path) "-file"))
 		  (system-remove what*))
@@ -256,35 +279,41 @@
 
 (define-public (converters-from-special fm suf tm?)
   (let* ((l1 (converters-from fm))
-         (l5 (list-filter l1 (lambda (s) (not (ahash-ref format-hidden s)))))
 	 (l2 (list-filter l1 (lambda (s) (string-ends? s suf))))
 	 (l3 (map (lambda (s) (string-drop-right s (string-length suf))) l2))
-	 (l4 (if tm? l3 (list-filter l3
-                                     (lambda (s)
-                                       (and
-                                         (not (ahash-ref format-hidden s))
-                                         (!= s "texmacs")))))))
+         (l3 (list-filter l3 (lambda (s) (not (ahash-ref format-hidden s)))))
+         (l4 (if tm? l3 (list-filter l3 (lambda (s) (!= s "texmacs"))))))
     (list-sort l4 format<=?)))
 
 (define-public (converters-to-special fm suf tm?)
   (let* ((l1 (converters-to fm))
 	 (l2 (list-filter l1 (lambda (s) (string-ends? s suf))))
 	 (l3 (map (lambda (s) (string-drop-right s (string-length suf))) l2))
-	 (l4 (if tm? l3 (list-filter l3
-                                     (lambda (s)
-                                       (and
-                                         (not (ahash-ref format-hidden s))
-                                         (!= s "texmacs")))))))
+         (l3 (list-filter l3 (lambda (s) (not (ahash-ref format-hidden s)))))
+         (l4 (if tm? l3 (list-filter l3 (lambda (s) (!= s "texmacs"))))))
     (list-sort l4 format<=?)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Other useful subroutines
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define-public (tmfile-get doc what)
+  (and (tm-func? doc 'document)
+       (with val (assoc-ref (map tm->list (tm-children doc)) what)
+         (if (pair? val) (set! val (car val)))
+         val)))
+
+(define-public (tmfile-set doc what val)
+  (and (tm-func? doc 'document)
+       (with l (reverse (map tm->list (tm-children doc)))
+         (set! l (assoc-set! l what (list val)))
+         (cons 'document (reverse l)))))
+
 (define-public (tmfile? doc)
-  (and (tmfile-extract doc 'TeXmacs) (tmfile-extract doc 'body)))
+  (and (tmfile-get doc 'TeXmacs) (tmfile-get doc 'body)))
 
 (define-public (tmfile-extract doc what)
+  ;; FIXME: use tmfile-get instead whenever possible
   (if (tree? doc) (set! doc (tree->stree doc)))
   (and (func? doc 'document)
        (with val (assoc-ref (cdr doc) what)
@@ -292,6 +321,14 @@
          (if (tree? val) (set! val (tree->stree val)))
          val)))
 
+(define-public (tmfile-assign doc what val)
+  ;; FIXME: use tmfile-set instead whenever possible
+  (if (tree? doc) (set! doc (tree->stree doc)))
+  (and (func? doc 'document)
+       (with l (reverse (cdr doc))
+         (set! l (assoc-set! l what (list (tm->tree val))))
+         (cons 'document (reverse l)))))
+
 (define (default-init var)
   ;; FIXME: should use C++ code
   (cond ((== var "mode") "text")
