@@ -19,23 +19,23 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind insert-math-menu
-  ("Formula" (make 'math))
+  ("Inline formula" (make 'math))
   (if (style-has? "env-math-dtd")
-      ("Equation" (make-equation*))
-      ("Equations" (make-eqnarray*))))
+      ("Displayed formula" (make-equation*))
+      ("Several equations" (make-eqnarray*))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Special mathematical text properties
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind math-special-format-menu
+  (-> "Display style"
+      ("On" (make-with "math-display" "true"))
+      ("Off" (make-with "math-display" "false")))
   (-> "Index level"
       ("Normal" (make-with "math-level" "0"))
       ("Script size" (make-with "math-level" "1"))
       ("Script script size" (make-with "math-level" "2")))
-  (-> "Display style"
-      ("On" (make-with "math-display" "true"))
-      ("Off" (make-with "math-display" "false")))
   (-> "Condensed"
       ("On" (make-with "math-condensed" "true"))
       ("Off" (make-with "math-condensed" "false"))))
@@ -66,7 +66,11 @@
       ("Font" (interactive open-font-selector)))
   (if (not (new-fonts?))
       (-> "Font" (link math-font-menu)))
-  (-> "Mathematics" (link math-special-format-menu))
+  (when (inside? 'table)
+      ("Cell" (open-cell-properties))
+      ("Table" (open-table-properties)))
+  ---
+  (link math-special-format-menu)
   ---
   (-> "Whitespace" (link horizontal-space-menu))
   (-> "Line break" (link line-break-menu))
@@ -77,8 +81,12 @@
 	  ---)
       (link color-menu))
   (-> "Adjust" (link adjust-menu))
+  (-> "Transform" (link linear-transform-menu))
   (-> "Specific" (link specific-menu))
-  (-> "Special" (link format-special-menu)))
+  (-> "Special" (link format-special-menu))
+  (-> "Font effects" (link text-font-effects-menu))
+  (assuming (== (get-preference "bitmap effects") "on")
+    (-> "Graphical effects" (link text-effects-menu))))
 
 (menu-bind math-format-menu
   (if (use-menus?)
@@ -95,9 +103,9 @@
   (-> "Large separator" (tile 8 (link middle-delimiter-menu)))
   (-> "Large closing bracket" (tile 8 (link right-delimiter-menu)))
   (-> "Big operator"
-      (tile 8 (link big-operator-menu)))
+      (tile 6 (link big-operator-menu)))
   ---
-  (-> "Binary operation"
+  (-> "Binary operator"
       (tile 8 (link binary-operation-menu)))
   (-> "Binary relation"
       (tile 8 (link binary-relation-menu-1))
@@ -234,26 +242,30 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind big-operator-menu
+  (symbol "<big-int-2>" (math-big-operator "int"))
+  (symbol "<big-iint-2>" (math-big-operator "iint"))
+  (symbol "<big-iiint-2>" (math-big-operator "iiint"))
+  (symbol "<big-idotsint-2>" (math-big-operator "idotsint"))
+  (symbol "<big-oint-2>" (math-big-operator "oint"))
+  (symbol "<big-oiint-2>" (math-big-operator "oiint"))
   (symbol "<big-sum-2>" (math-big-operator "sum"))
   (symbol "<big-prod-2>" (math-big-operator "prod"))
-  (symbol "<big-int-2>" (math-big-operator "int"))
-  (symbol "<big-oint-2>" (math-big-operator "oint"))
   (symbol "<big-amalg-2>" (math-big-operator "amalg"))
-  (symbol "<big-cap-2>" (math-big-operator "cap"))
-  (symbol "<big-cup-2>" (math-big-operator "cup"))
-  (symbol "<big-wedge-2>" (math-big-operator "wedge"))
-  (symbol "<big-vee-2>" (math-big-operator "vee"))
   (symbol "<big-odot-2>" (math-big-operator "odot"))
   (symbol "<big-oplus-2>" (math-big-operator "oplus"))
   (symbol "<big-otimes-2>" (math-big-operator "otimes"))
+  (symbol "<big-cap-2>" (math-big-operator "cap"))
+  (symbol "<big-cup-2>" (math-big-operator "cup"))
+  (symbol "<big-pluscup-2>" (math-big-operator "pluscup"))
   (symbol "<big-sqcap-2>" (math-big-operator "sqcap"))
   (symbol "<big-sqcup-2>" (math-big-operator "sqcup"))
+  (symbol "<big-box-2>" (math-big-operator "box"))
+  (symbol "<big-wedge-2>" (math-big-operator "wedge"))
+  (symbol "<big-vee-2>" (math-big-operator "vee"))
   (symbol "<big-curlywedge-2>" (math-big-operator "curlywedge"))
   (symbol "<big-curlyvee-2>" (math-big-operator "curlyvee"))
   (symbol "<big-triangleup-2>" (math-big-operator "triangleup"))
   (symbol "<big-triangledown-2>" (math-big-operator "triangledown"))
-  (symbol "<big-box-2>" (math-big-operator "box"))
-  (symbol "<big-pluscup-2>" (math-big-operator "pluscup"))
   (symbol "<big-parallel-2>" (math-big-operator "parallel"))
   (symbol "<big-interleave-2>" (math-big-operator "interleave")))
 
@@ -993,7 +1005,8 @@
 (menu-bind semantic-math-preferences-menu
   ("Semantic editing" (toggle-preference "semantic editing"))
   (when (== (get-preference "semantic editing") "on")
-    ("Semantic selections" (toggle-preference "semantic selections"))))
+    ("Semantic selections" (toggle-preference "semantic selections")))
+  (if #t ("Semantic correctness" (toggle-preference "semantic correctness"))))
 
 (menu-bind semantic-annotation-menu
   ("Ordinary symbol" (make 'math-ordinary))
@@ -1021,10 +1034,10 @@
   ("Other" (make 'syntax)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; The Mathematics menu
+;; Menu for inserting mathematical markup
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(menu-bind math-menu
+(menu-bind math-insert-menu
   ("Fraction" (make-fraction))
   ("Square root" (make-sqrt))
   ("N-th root" (make-var-sqrt))
@@ -1052,14 +1065,20 @@
       ("Grave" (make-wide "<grave>"))
       ("Dot" (make-wide "<dot>"))
       ("Two dots" (make-wide "<ddot>"))
+      ("Three dots" (make-wide "<dddot>"))
+      ("Four dots" (make-wide "<ddddot>"))
       ("Circle" (make-wide "<abovering>"))
       ---
       ("Overbrace" (make-wide "<wide-overbrace>"))
       ("Underbrace" (make-wide "<wide-underbrace*>"))
+      ("Round overbrace" (make-wide "<wide-poverbrace>"))
+      ("Round underbrace" (make-wide "<wide-punderbrace*>"))
       ("Square overbrace" (make-wide "<wide-sqoverbrace>"))
       ("Square underbrace" (make-wide "<wide-squnderbrace*>"))
+      ---
       ("Right arrow" (make-wide "<wide-varrightarrow>"))
       ("Left arrow" (make-wide "<wide-varleftarrow>"))
+      ("Left-right arrow" (make-wide "<wide-varleftrightarrow>"))
       ("Wide bar" (make-wide "<wide-bar>")))
   (-> "Accent below"
       ("Tilda" (make-wide-under "~"))
@@ -1074,28 +1093,34 @@
       ("Grave" (make-wide-under "<grave>"))
       ("Dot" (make-wide-under "<dot>"))
       ("Two dots" (make-wide-under "<ddot>"))
+      ("Three dots" (make-wide-under "<dddot>"))
+      ("Four dots" (make-wide-under "<ddddot>"))
       ("Circle" (make-wide-under "<abovering>"))
       ---
       ("Overbrace" (make-wide-under "<wide-overbrace*>"))
       ("Underbrace" (make-wide-under "<wide-underbrace>"))
+      ("Round overbrace" (make-wide-under "<wide-poverbrace*>"))
+      ("Round underbrace" (make-wide-under "<wide-punderbrace>"))
       ("Square overbrace" (make-wide-under "<wide-sqoverbrace*>"))
       ("Square underbrace" (make-wide-under "<wide-squnderbrace>"))
+      ---
       ("Right arrow" (make-wide-under "<wide-varrightarrow>"))
       ("Left arrow" (make-wide-under "<wide-varleftarrow>"))
+      ("Left-right arrow" (make-wide-under "<wide-varleftrightarrow>"))
       ("Wide bar" (make-wide-under "<wide-bar>")))
   (-> "Symbol" (link symbol-menu))
   (-> "Textual operator" (link textual-operator-menu))
   (if (== (get-preference "semantic editing") "on")
-      (-> "Semantics" (link semantic-annotation-menu)))
+      (-> "Semantics" (link semantic-annotation-menu))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; The Mathematics menu
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(menu-bind math-menu
+  (link math-insert-menu)
   ---
-  ("Text" (make 'text))
-  (-> "Table" (link insert-table-menu))
-  (-> "Image" (link insert-image-menu))
-  (-> "Link" (link insert-link-menu))
-  (if (detailed-menus?)
-      (if (style-has? "std-fold-dtd")
-          (-> "Fold" (link insert-fold-menu)))
-      (-> "Animation" (link insert-animation-menu))))
+  (link texmacs-insert-menu))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Icons for modifying mathematical text properties
@@ -1111,25 +1136,26 @@
       ("Small inline" (make-with "math-display" "false"))
       ("Large displayed" (make-with "math-display" "true"))
       ---
-      (group "Spacing")
-      ("Normal" (make-with "math-condensed" "false"))
-      ("Condensed" (make-with "math-condensed" "true"))
-      ---
       (group "Size")
       ("Normal" (make-with "math-level" "0"))
       ("Script size" (make-with "math-level" "1"))
-      ("Script script size" (make-with "math-level" "2"))))
+      ("Script script size" (make-with "math-level" "2"))
+      ---
+      (group "Spacing")
+      ("Normal" (make-with "math-condensed" "false"))
+      ("Condensed" (make-with "math-condensed" "true"))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Icons for math mode
+;; Icons for inserting mathematical markup
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(menu-bind math-icons
+(menu-bind math-insert-icons
   (=> (balloon (icon "tm_fraction.xpm") "Insert a fraction")
       ("Standard fraction" (make-fraction))
       ("Small inline fraction" (make 'tfrac))
       ("Large displayed fraction" (make 'dfrac))
-      ("Slash" (make 'frac*)))
+      ("Slashed fraction" (make 'frac*))
+      ("Continued fraction" (make 'cfrac)))
   (=> (balloon (icon "tm_root.xpm") "Insert a root")
       ("Square root" (make-sqrt))
       ("Multiple root" (make-var-sqrt)))
@@ -1142,7 +1168,7 @@
       ("Superscript above" (make-above)))
   /
   (=> (balloon (icon "tm_bigop.xpm") "Insert a big operator")
-      (tile 8 (link big-operator-menu)))
+      (tile 6 (link big-operator-menu)))
   (=> (balloon (icon "tm_bigaround.xpm") "Insert large delimiters")
       (tile 8 (link large-delimiter-menu))
       ---
@@ -1186,7 +1212,7 @@
       ---
       (tile 6 (link dots-menu)))
   /
-  (=> (balloon (icon "tm_greek.xpm") "Insert a greek character")
+  (=> (balloon (icon "tm_greek_char.xpm") "Insert a greek character")
       (tile 8 (link lower-greek-menu))
       ---
       (tile 8 (link upper-greek-menu)))
@@ -1225,7 +1251,14 @@
   (if (== (get-preference "semantic editing") "on")
       (=> (balloon (icon "tm_math_syntax.xpm")
                    "Specify semantics of a symbol or formula")
-          (link semantic-annotation-menu)))
+          (link semantic-annotation-menu))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Icons for math mode
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(menu-bind math-icons
+  (link math-insert-icons)
   (link texmacs-insert-icons))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1242,11 +1275,11 @@
 
 (tm-define (focus-tag-name l)
   (:require (== l 'math))
-  "Formula")
+  "Inline formula")
 
 (tm-define (focus-tag-name l)
   (:require (in? l '(equation equation*)))
-  "Equation")
+  "Displayed formula")
 
 (tm-define (focus-tag-name l)
   (:require (in? l '(eqnarray eqnarray*)))
@@ -1262,8 +1295,8 @@
 
 (tm-menu (focus-variant-menu t)
   (:require (tree-in? t '(math equation equation*)))
-  ("Formula" (variant-formula t))
-  ("Equation" (variant-equation t)))
+  ("Inline formula" (variant-formula t))
+  ("Displayed formula" (variant-equation t)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Script focus menus
@@ -1383,8 +1416,23 @@
           (alternate-second? (focus-tree)))
    (alternate-toggle (focus-tree))))
 
+(tm-menu (focus-extra-menu t)
+  (:require (tree-in? t '(left mid right around around*)))
+  ---
+  ("Increase size" (geometry-up))
+  ("Decrease size" (geometry-down))
+  ("Default size" (geometry-reset)))
+
 (tm-menu (focus-toggle-icons t)
   (:require (tree-in? t '(around around*)))
   ((check (balloon (icon "tm_large_around.xpm") "Large brackets") "v"
           (alternate-second? (focus-tree)))
-   (alternate-toggle (focus-tree))))
+   (alternate-toggle (focus-tree)))
+  ;; TODO: create suitable icons
+  ;;((balloon (icon "tm_plus.xpm") "Increase bracket size")
+  ;; (geometry-up))
+  ;;((balloon (icon "tm_minus.xpm") "Decrease bracket size")
+  ;; (geometry-down))
+  ;;((balloon (icon "tm_reset.xpm") "Reset to default bracket size")
+  ;; (geometry-reset))
+  )
